/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the diagram

#ifndef __DIAGRAM_GRAPHIC_H__
#define __DIAGRAM_GRAPHIC_H__

#include "orderedset.h"
#include <set>

class OrderData;

class LIBLATVIS_API Diagram_Graphic {
 public:
  enum generation_method_t {DIAGRAM_LEVEL, DIAGRAM_GEOMETRIC, DIAGRAM_LAYER};
  enum dla_l_t {DLA_L_LONGEST_PATH, DLA_L_COFFMAN_GRAHAM};
  typedef unsigned int dla_cgw_t;
  enum dla_dsdt_t {DLA_DD_TREE, DLA_DD_LIST, DLA_DD_ARRAY};
  enum dla_cr_t {DLA_CR_ADJACENT_EXCHANGE, DLA_CR_SPLIT, DLA_CR_BARYCENTER,
		 DLA_CR_MEDIAN};
  struct generationParams_t {
    generation_method_t generation_method;
    bool dle_fmin;
    bool dg_fgreat;
    bool dla_fmax;
    dla_l_t dla_l;
    dla_cgw_t dla_cgw;
    dla_dsdt_t dla_dsdt;
    dla_cr_t dla_cr;
  };
 protected:
  OrderedSet *orderedset;
  OrderData *data;
  set<Vertex> verteces;
  OrderData *dummy_chains_data;
  OrderedSet::data_t dummy_chains_datatype;
  Coord_t min_x, max_x, min_y, max_y;
  generationParams_t generationParams;
 public:
  Diagram_Graphic(OrderedSet *os, OrderData *d,
		  OrderedSet::data_t dummy_chains_data_type);
  ~Diagram_Graphic();
  Diagram_Graphic *getCopy(OrderData *new_data);
  // returns: the object copy (or NULL)
  generationParams_t getGenerationParams() { return generationParams; }
  // returns: diagram generation parameters
  OrderedSet::data_t getDummyChainsDataType() { 
    return dummy_chains_datatype;
  }
  // returns: dummy chains data representation
  bool generateDiagram(generationParams_t params);
  // generates diagram using the parameters in params
  // returns: if successful (not)
  Vertex *addVertex(Vertex v);
  // adds vertex v
  // returns: new vertex, or NULL
  bool deleteVertex(Vertex v);
  // deletes vertex v
  // returns: if successful (not)
  void getBox(Coord_t &x1, Coord_t &y1, Coord_t &x2, Coord_t &y2) { 
    x1 = min_x; y1 = min_y; x2 = max_x; y2 = max_y;
  }
  // returns: diagram box
  Vertex *getVertex(Vertex v);
  // returns: the true vertex v, or NULL
  Vertex *getVertex(Element x);
  // returns: the vertex for element x, or NULL
  void getVerteces(set<Vertex> &s) { s = verteces; }
  // returns: all verteces in s
  void clearDiagram();
  // clears the diagram
  Vertex *getVertexInBox(Coord_t x, Coord_t y, Coord_t d);
  // returns: the vertex in the box (x-d, y-d), (x+d, y+d), or NULL
  Vertex *updateVertex(Vertex v);
  // updates certain vertex to be the vertex v
  // returns: the updated vertex
  Element *getElement(Vertex v);
  // returns: element corresponding to vertex v, or NULL
  bool isEdge(Vertex vx, Vertex vy);
  // returns: if there is an edge between verteces vx and vy
  bool addDummyEdge(Vertex vx, Vertex vy);
  // adds edge between dummy (or mixed) verteces vx and vy
  // returns: if successful (not)
  bool deleteDummyEdge(Vertex vx, Vertex vy);
  // deletes edge between dummy (or mixed) verteces vx and vy
  // returns: if successful (not)
};

#endif
