/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// classes of the vertex and the element

#ifndef __ELEMENT_H__
#define __ELEMENT_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif
#ifdef WIN32_DLL
#include "dll.h"
#else
#define LIBLATVIS_API
#endif
#include <string>

// MSVC wants this
using namespace std;

typedef unsigned short ElementId;
const ElementId NoElement = 0;

typedef string ElementName;

typedef short Coord_t;
#ifdef HAVE_LIMITS_H
#define COORD_MIN SHRT_MIN
#define COORD_MAX SHRT_MAX
#else
#define COORD_MIN -32768
#define COORD_MAX 32767
#endif

class LIBLATVIS_API Vertex {
 public:
  enum {HIDDEN = 1, HIDDEN_LABEL = 2, SELECTED = 4, MARKED = 8} properties_t;
  ElementId element;
  ElementId dummy;
  Coord_t x;
  Coord_t y;
  unsigned long properties;
  
  Vertex(ElementId _element, ElementId _dummy = NoElement, 
	 Coord_t _x = 0, Coord_t _y = 0)
    {
      element = _element; dummy = _dummy; x = _x; y = _y;
      properties = 0;
    }
  // comparison function for STL
  bool
    operator <(const Vertex &v) const
    {
      if (element < v.element) return true;
      if (v.element < element) return false;
      return ((element == NoElement) && (v.element == NoElement) && 
	      (dummy < v.dummy));
    }
  bool
    operator >(const Vertex &v) const
    {
      return (v < *this);
    }
  bool
    operator ==(const Vertex &v) const
    {
      return (!((*this < v) || (v < *this)));
    }
  bool
    operator !=(const Vertex &v) const
    {
      return ((*this < v) || (v < *this));
    }
};
  
class LIBLATVIS_API Element {
 public:
  ElementId id;
  ElementName name;

  Element(ElementId _id, ElementName _name = "")
    {
      id = _id; name = _name;
    }
  // comparison function for STL
  bool
    operator <(const Element &e) const
    {
      return (id < e.id);
    }
  bool
    operator >(const Element &e) const
    {
      return (e < *this);
    }
  bool
    operator ==(const Element &e) const
    {
      return (!((*this < e) || (e < *this)));
    }
  bool
    operator !=(const Element &e) const
    {
      return ((*this < e) || (e < *this));
    }
};

#endif
