/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <libxml/tree.h>
#include "saver.h"
#include "diagram_graphic.h"
#include <stdio.h>

bool
FileSaver::saveToXML(string filename)
{
  bool result;
  xmlDocPtr doc;
  xmlNodePtr root_node, elements_node = NULL, element_node, vertex_node,
    orders_node = NULL, order_node, diagram_node = NULL, dummies_node = NULL,
    dummy_node, edges_node = NULL, edge_node;
  Vertex *vertex;
  set<Element> elements;
  set<Element>::iterator iter_se1, iter_se2;
  set<Vertex> verteces;
  set<Vertex>::iterator iter_sv1, iter_sv2;
  char coord_str[4], elem_id_str[5], dummy_id_str[5];
  
  doc = xmlNewDoc((xmlChar *)"1.0");
  root_node = xmlNewDocNode(doc, NULL, (xmlChar *)"ordered_set", NULL);
  switch (orderedset->getDataType()) {
  case OrderedSet::DATA_TREE: 
    xmlNewProp(root_node, (xmlChar *)"data_type", (xmlChar *)"tree"); break;
  case OrderedSet::DATA_LIST:
    xmlNewProp(root_node, (xmlChar *)"data_type", (xmlChar *)"list"); break;
  case OrderedSet::DATA_ARRAY:
    xmlNewProp(root_node, (xmlChar *)"data_type", (xmlChar *)"array");
  }
  xmlDocSetRootElement(doc, root_node);
  orderedset->getElements(elements);
  for (iter_se1 = elements.begin(); iter_se1 != elements.end(); iter_se1++) {
    if (!elements_node)
      elements_node = 
	xmlNewChild(root_node, NULL, (xmlChar *)"elements", NULL);  
    element_node = 
      xmlNewChild(elements_node, NULL, (xmlChar *)"element", NULL);
    sprintf(elem_id_str, "%i", (*iter_se1).id);
    xmlNewProp(element_node, (xmlChar *)"id", (xmlChar *)elem_id_str);
    xmlNewProp(element_node, (xmlChar *)"name", 
	       (xmlChar *)(*iter_se1).name.c_str());
    vertex = orderedset->getDiagram()->getVertex(*iter_se1);
    if (vertex) {
      vertex_node = xmlNewChild(element_node, NULL, (xmlChar *)"vertex", NULL);
      if (vertex->dummy != NoElement) {
	sprintf(dummy_id_str, "%i", vertex->dummy);
	xmlNewProp(vertex_node, (xmlChar *)"dummy_id",
		   (xmlChar *)dummy_id_str);
      }
      sprintf(coord_str, "%i", vertex->x);
      xmlNewChild(vertex_node, NULL, (xmlChar *)"x", (xmlChar *)coord_str);
      sprintf(coord_str, "%i", vertex->y);
      xmlNewChild(vertex_node, NULL, (xmlChar *)"y", (xmlChar *)coord_str);
    }
    for (iter_se2 = elements.begin(); iter_se2 != elements.end();
	 iter_se2++) {
      if (*iter_se2 == *iter_se1) continue;
      if (orderedset->less(*iter_se1, *iter_se2, false)) {
	if (!orders_node)
	  orders_node = 
	    xmlNewChild(root_node, NULL, (xmlChar *)"orders", NULL);
	order_node = xmlNewChild(orders_node, NULL, (xmlChar *)"order", NULL);
	element_node = 
	  xmlNewChild(order_node, NULL, (xmlChar *)"pred", NULL);
	sprintf(elem_id_str, "%i", (*iter_se1).id);
	xmlNewProp(element_node, (xmlChar *)"id", (xmlChar *)elem_id_str);
	element_node = 
	  xmlNewChild(order_node, NULL, (xmlChar *)"succ", NULL);
	sprintf(elem_id_str, "%i", (*iter_se2).id);
	xmlNewProp(element_node, (xmlChar *)"id", (xmlChar *)elem_id_str);
      }
    }
  }
  orderedset->getDiagram()->getVerteces(verteces);
  for (iter_sv1 = verteces.begin(); iter_sv1 != verteces.end(); iter_sv1++) {
    if ((*iter_sv1).dummy == NoElement) continue;
    if ((*iter_sv1).element == NoElement) { // dummy vertex
      if (!diagram_node)
	diagram_node = 
	  xmlNewChild(root_node, NULL, (xmlChar *)"diagram", NULL);  
      if (!dummies_node)
	dummies_node = 
	  xmlNewChild(diagram_node, NULL, (xmlChar *)"dummies", NULL);  
      dummy_node = 
	xmlNewChild(dummies_node, NULL, (xmlChar *)"dummy", NULL);
      sprintf(dummy_id_str, "%i", (*iter_sv1).dummy);
      xmlNewProp(dummy_node, (xmlChar *)"id", (xmlChar *)dummy_id_str);
      sprintf(coord_str, "%i", (*iter_sv1).x);
      xmlNewChild(dummy_node, NULL, (xmlChar *)"x", (xmlChar *)coord_str);
      sprintf(coord_str, "%i", (*iter_sv1).y);
      xmlNewChild(dummy_node, NULL, (xmlChar *)"y", (xmlChar *)coord_str);
    }
    for (iter_sv2 = verteces.begin(); iter_sv2 != verteces.end();
	 iter_sv2++) {
      if (((*iter_sv2).dummy == NoElement) || (*iter_sv1 == *iter_sv2) ||
	  (((*iter_sv1).element != NoElement) && 
	   ((*iter_sv2).element != NoElement)))
	continue; // at least one dummy vertex
      if (orderedset->getDiagram()->isEdge(*iter_sv1, *iter_sv2)) {
	if (!edges_node)
	  edges_node = 
	    xmlNewChild(diagram_node, NULL, (xmlChar *)"edges", NULL);  
	edge_node = xmlNewChild(edges_node, NULL, (xmlChar *)"edge", NULL);
	dummy_node = 
	  xmlNewChild(edge_node, NULL, (xmlChar *)"dummy", NULL);
	sprintf(dummy_id_str, "%i", (*iter_sv1).dummy);
	xmlNewProp(dummy_node, (xmlChar *)"id", (xmlChar *)dummy_id_str);
	dummy_node = 
	  xmlNewChild(edge_node, NULL, (xmlChar *)"dummy", NULL);
	sprintf(dummy_id_str, "%i", (*iter_sv2).dummy);
	xmlNewProp(dummy_node, (xmlChar *)"id", (xmlChar *)dummy_id_str);
      }
    }
  }
  result = (xmlSaveFormatFile(filename.c_str(), doc, true) != -1);
  xmlFreeDoc(doc);
  return result;
}

bool
FileSaver::saveToMetaPost(string filename)
{
  FILE *file = NULL;
  set<Vertex> verteces;
  set<Vertex>::iterator iter_s1, iter_s2;
  Element *elem = NULL;
  int vx1, vy1, vx2, vy2, xoff = 0, yoff = 0;
  Coord_t x1 = 0, y1 = 0, x2 = 0, y2 = 0;
  const int diagram_vertex_size = 10;
  int dvs2;
  string color;

  dvs2 = diagram_vertex_size >> 1;  
  if (filename == "-") file = stdout;
  else file = fopen(filename.c_str(), "w");
  if (!file) return false;

  /*
    fprintf(file, "prologues := 1;\n\
    \n");
  */

  /*
    fprintf(file, "defaultfont := \"cmr10\";\n\
    \n"); // czech font is csr
  */
  
  /*
    fprintf(file, "%%verbatimtex \\font\\muj = cmr10 \\muj etex;\n\
    \n");
  */

  fprintf(file, "vardef vertex (expr z, c) =\n\
  pickup pencircle scaled diagram_vertex_size;\n\
  draw z withcolor c;\n\
enddef;\n\
\n\
vardef vertexwithlabel (expr z, c, t) (suffix pos) =\n\
  picture lab;\n\
  drawoptions(withcolor c);\n\
  pickup pencircle scaled diagram_vertex_size;\n\
  draw z;\n\
  pickup defaultpen;\n\
  lab = thelabel.pos(t, z);\n\
  filldraw bbox lab withcolor white;\n\
  draw bbox lab;\n\
  draw lab;\n\
enddef;\n\
\n\
beginfig(1);\n\
  u := 1pt;\n\
  diagram_vertex_size := %ipt;\n\
  labeloffset := diagram_vertex_size;\n", diagram_vertex_size);

  orderedset->getDiagram()->getVerteces(verteces);
  if (!verteces.empty()) {
    orderedset->getDiagram()->getBox(x1, y1, x2, y2);
    xoff = x1 - dvs2; yoff = y1 - dvs2;
  }
  for (iter_s1 = verteces.begin(); iter_s1 != verteces.end(); iter_s1++) {
    vx1 = (*iter_s1).x - xoff; vy1 = (*iter_s1).y - yoff;
    for (iter_s2 = verteces.begin(); iter_s2 != verteces.end(); iter_s2++) {
      if (*iter_s1 == *iter_s2) continue;
      if (orderedset->getDiagram()->isEdge(*iter_s1, *iter_s2)) {
        vx2 = (*iter_s2).x - xoff; vy2 = (*iter_s2).y - yoff;
        fprintf(file, "  draw (%iu, %iu) -- (%iu, %iu);\n",
		vx1, vy1, vx2, vy2);
      }
    }
  }
  for (iter_s1 = verteces.begin(); iter_s1 != verteces.end(); iter_s1++) {
    vx1 = (*iter_s1).x - xoff; vy1 = (*iter_s1).y - yoff;
    elem = orderedset->getDiagram()->getElement(*iter_s1);
    if (elem) {
      color = "black";
      if ((*iter_s1).properties & Vertex::SELECTED) color += "+green";
      if ((*iter_s1).properties & Vertex::MARKED) color += "+blue";
      if ((*iter_s1).properties & Vertex::HIDDEN_LABEL)
	//fprintf(file, "  vertex((%iu, %iu), \"%s\");\n",
	//      vx1, vy1, elem->name.c_str());
	
	fprintf(file, "  vertex((%iu, %iu), %s);\n", 
		vx1, vy1, color.c_str());
	
      else
	
	fprintf(file, "  vertexwithlabel((%iu, %iu), %s, btex $%s$ etex, urt);\n",
		vx1, vy1, color.c_str(), elem->name.c_str());
      
    }
  }

  fprintf(file, "endfig;\n\
\n\
bye\n");
  
  if (file != stdout) fclose(file);
  return true;
}

bool
FileSaver::saveToPostScript(string filename)
{
  FILE *file = NULL;
  set<Vertex> verteces;
  set<Vertex>::iterator iter_s1, iter_s2;
  Element *elem = NULL;
  int vx1, vy1, vx2, vy2, xoff = 0, yoff = 0;
  Coord_t x1 = 0, y1 = 0, x2 = 0, y2 = 0;
  const int diagram_vertex_size = 10, font_width = 10, font_height = 10,
    d = 2, e = 3;
  int dvs2, label_size;
  bool first_thing = true, color_change;
  int color_r = 0, color_g = 0, color_b = 0;
  
  dvs2 = diagram_vertex_size >> 1;
  if (filename == "-") file = stdout;
  else file = fopen(filename.c_str(), "w");
  if (!file) return false;

  fprintf(file, "%%!PS-Adobe-3.0 EPSF-3.0\n");
  orderedset->getDiagram()->getVerteces(verteces);
  if (!verteces.empty()) {
    orderedset->getDiagram()->getBox(x1, y1, x2, y2);
    xoff = x1 - dvs2; yoff = y1 - dvs2;
    for (iter_s1 = verteces.begin(); iter_s1 != verteces.end(); iter_s1++) {
      elem = orderedset->getDiagram()->getElement(*iter_s1);
      if (elem) {
	label_size = elem->name.length() * (font_width - e);
	if (((*iter_s1).x + label_size) > x2) x2 = (*iter_s1).x + label_size;
      }
    }
    fprintf(file, "%%%%BoundingBox: %i %i %i %i\n", 0, 0,
	    x2 + dvs2 + d - xoff, y2 + dvs2 + font_height + d - yoff);
  }

  fprintf(file, "%%%%Creator: %s\n\
%%%%CreationDate: 2003.03.05:1730\n\
%%%%Pages: 1\n\
%%%%DocumentFonts: cmr10\n\
 /cmr10 /cmr10 def\n\
/fshow {exch findfont exch scalefont setfont show}bind def\n\
%%%%EndProlog\n\
%%%%Page: 1 1\n", PROG_NAME);
  // TODO: CreationDate

  for (iter_s1 = verteces.begin(); iter_s1 != verteces.end(); iter_s1++) {
    vx1 = (*iter_s1).x - xoff; vy1 = (*iter_s1).y - yoff;
    for (iter_s2 = verteces.begin(); iter_s2 != verteces.end(); iter_s2++) {
      if (*iter_s1 == *iter_s2) continue;
      if (orderedset->getDiagram()->isEdge(*iter_s1, *iter_s2)) {
        vx2 = (*iter_s2).x - xoff; vy2 = (*iter_s2).y - yoff;
	if (first_thing) { 
	  fprintf(file,
		  " 0 0.5 dtransform truncate idtransform setlinewidth pop");

	  fprintf(file, " [] 0 setdash\n\
 1 setlinecap 1 setlinejoin 10 setmiterlimit\n");

	  first_thing = false;
	}

	fprintf(file, "newpath %i %i moveto\n\
%i %i lineto stroke\n",
		vx1, vy1, vx2, vy2);

      }
    }
  }
  for (iter_s1 = verteces.begin(); iter_s1 != verteces.end(); iter_s1++) {
    vx1 = (*iter_s1).x - xoff; vy1 = (*iter_s1).y - yoff;
    elem = orderedset->getDiagram()->getElement(*iter_s1);
    if (elem) {
      vx1 += dvs2; vy1 += dvs2;
      label_size = elem->name.length() * (font_width - e);
      if (color_r || color_g || color_b) color_change = true;
      else color_change = false;
      color_r = color_g = color_b = 0;
      if ((*iter_s1).properties & Vertex::SELECTED) color_g += 1;
      if ((*iter_s1).properties & Vertex::MARKED) color_b += 1;
      if (color_r || color_g || color_b)
	fprintf(file, " %i %i %i setrgbcolor", color_r, color_g, color_b);
      else if (color_change) fprintf(file, " 0 setgray");
      fprintf(file, " 0 %i dtransform truncate idtransform setlinewidth pop",
	      diagram_vertex_size);
      if (first_thing) {

	fprintf(file, " [] 0 setdash\n\
 1 setlinecap 1 setlinejoin 10 setmiterlimit\n");

	first_thing = false;
      }
      else fprintf(file, "\n");
      fprintf(file, "newpath %i %i moveto 0 0 rlineto stroke\n",
	      vx1 - dvs2, vy1 - dvs2); // point
      if (!((*iter_s1).properties & Vertex::HIDDEN_LABEL)) {

	fprintf(file," 1 setgray 0 0.5 dtransform truncate idtransform setlinewidth pop\n\
newpath %i %i moveto\n\
%i %i lineto\n\
%i %i lineto\n\
%i %i lineto\n\
 closepath\n\
gsave fill grestore stroke\n",
		// background filled box
		vx1, vy1, vx1 + label_size + d, vy1, vx1 + label_size + d,
		vy1 + font_height + d, vx1, vy1 + font_height + d);
      
	if (color_r || color_g || color_b)
	  fprintf(file, "%i %i %i setrgbcolor\n", color_r, color_g, color_b);
	else fprintf(file, "0 setgray\n");
	
	fprintf(file, "newpath %i %i moveto\n\
%i %i lineto\n\
%i %i lineto\n\
%i %i lineto\n\
 closepath stroke\n\
%i %i moveto\n\
(%s) cmr10 %i fshow\n",
		// box
		vx1, vy1, vx1 + label_size + d, vy1, vx1 + label_size + d,
		vy1 + font_height + d, vx1, vy1 + font_height + d,
		vx1 + d, vy1 + d, elem->name.c_str(), font_width); // label
      }
    }
  }
  
  fprintf(file, "showpage\n\
%%%%EOF\n");
  
  if (file != stdout) fclose(file);
  return true;
}

FileSaver::FileSaver(OrderedSet *os)
{
  orderedset = os;
}

FileSaver::error_t
FileSaver::save(string filename, type_t type)
{
  string::size_type pos;
  string stype = "";
  FILE *file = NULL;
  bool save_ok;
  
  if (type == NOTYPE) {
    if (filename == "-") type = XML;
    else {
      pos = filename.rfind('.');
      if (pos != filename.npos) stype = filename.substr(pos);
      if (stype == ".lat") type = XML;
      else if (stype == ".mp") type = MetaPost;
      else if (stype == ".eps") type = PostScript;
    }
  }
  if (type == NOTYPE) return ETYPE;
  if (filename != "-") {
    file = fopen(filename.c_str(), "w");
    if (!file) return EACCESS;
    else {
      if (putc('0', file) == EOF) { fclose(file); return EACCESS; }
      fclose(file);
    }
  }
  switch (type) {
  case XML: 
    save_ok = saveToXML(filename);
    if (save_ok) { 
      orderedset->fileSaved(); orderedset->setFilename(filename);
    }
    break;
  case MetaPost: save_ok = saveToMetaPost(filename); break;
  case PostScript: save_ok = saveToPostScript(filename); break;
  default:;
  }
  if (!save_ok) return ESAVE;
  return EOK;
}
