/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else
#include "getopt.h"
#endif
#include "options.h"

void
Options::parse()
{
  int o, arg;
  struct option long_options[] = {
    {"help", no_argument, NULL, 'h'},
    {"version", no_argument, NULL, 'v'},
    {"diagram", no_argument, NULL, 'd'},
    {"method", required_argument, NULL, 'm'},
    {"save", no_argument, NULL, 's'},
    {0, 0, 0, 0}
  };
  
  help = version = diagram = false;
  diagram_method = OPTION_DIAGRAM_METHOD_NONE; save = false;
  file_index = 0;
  opterr = 0;
  while ((o = getopt_long(argc, argv, "hvdm:s", long_options, NULL)) != -1) {
    switch (o) {
    case 'h': help = true; return;
    case 'v': version = true; return;
    case 'd': diagram = true; break;
    case 'm':
      arg = atoi(optarg); 
      if ((arg >= 1) && (arg <= 3)) diagram_method = (diagram_method_t)arg;
      break;
    case 's': save = true; break;
    case ':':
      fprintf(stderr, "%s: argument to `-%c' is missing\n", PACKAGE,
              optopt);
      break;
    default: ;
    }
  }
  if (optind < argc) file_index = optind;
}

Options::Options(int argc_, char *argv_[])
{ 
  argc = argc_; argv = argv_;
  help = version = diagram = false;
  diagram_method = OPTION_DIAGRAM_METHOD_NONE; save = false;
  file_index = 0;
  parse();
}

string
Options::getNextFilename()
{
  if ((file_index < argc) && (file_index)) {
    file_index++;
    return argv[file_index - 1];
  }
  return "";
}

void
Options::printHelp()
{
  printf("\nUsage: %s [option ... ] [file ... ]\n\n", PACKAGE);
  printf("Options:\n");
  printf("  -h           --help      Output this help.\n");
  printf("  -v           --version   Output version information.\n");
  printf("  -d           --diagram   Create the diagram on load.\n");
  printf("  -m <method>  --method    Select diagram generation method.\n");
  printf("                           (1 - level, 2 - geometric (default), 3 - layer)\n");
  printf("  -s           --save      Save loaded file(s). (GUI is not run.)\n\n");
}

void
Options::printVersion()
{
  printf("%s version %s\n", PROG_NAME, VERSION);
}
