/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "appwindow.h"
#include "config.h"
#include "appfilemenu.h"
#include "appbuttons.h"
#include "dialogs.h"
#include "i18n.h"
#include <qlayout.h>
#include <qmenubar.h>
#include <stdio.h>

bool
AppWindow::reallyClose()
{
  QuestionDialog *question_dialog;
  bool answer = true;
  
  if (!orderedsets->areAllSetsSaved()) {
    question_dialog = 
      new QuestionDialog(QString::fromLocal8Bit(_("Really quit?")), 
			 QString::fromLocal8Bit(_("Some files unsaved.")), 
			 QString::fromLocal8Bit(_("Quit the")) + " " + 
			 title + "?");
    answer = question_dialog->exec();
    delete question_dialog;
  }
  return answer;
}

void
AppWindow::closeEvent(QCloseEvent *ev)
{
  if (reallyClose()) QWidget::closeEvent(ev);
}

void
AppWindow::help()
{

}

void
AppWindow::about()
{
  about_dialog->exec();
}

AppWindow::AppWindow(OrderedSets *oss) : QWidget()
{
  QVBoxLayout *vbox;
  QMenuBar *menubar;
  AppFileMenu *filemenu;
  QPopupMenu *helpmenu;
  
  orderedsets = oss;
  // TODO: will be in preferences
  copy_orderedset = new OrderedSet(OrderedSet::DATA_ARRAY);
  title = PROG_NAME;
  about_dialog = new AboutDialog;
  // set title
  setCaption(title);
  // disable shrinking
  // menu
  menubar = new QMenuBar(this);
  // shadow in
  // file
  filemenu = new AppFileMenu(this, oss);
  menubar->insertItem(QString::fromLocal8Bit(_("&File")), filemenu);
  //help
  helpmenu = new QPopupMenu(this);
  menubar->insertItem(QString::fromLocal8Bit(_("&Help")), helpmenu);
  // tearoff
  helpmenu->insertTearOffHandle();
  // help
  //helpmenu->insertItem(_("Help..."), this, SLOT(help()), Key_F1);
  // about
  helpmenu->insertItem(QString::fromLocal8Bit(_("About...")), this, 
		       SLOT(about()));
  helpmenu->show();
  menubar->show();
  // buttons
  buttons = new AppButtons(this);
  buttons->show();
  vbox = new QVBoxLayout(this);
  // menubar may not grow
  vbox->setMenuBar(menubar);
  // buttons may
  vbox->addWidget(buttons, 1);
  show();
}

AppWindow::~AppWindow()
{
  delete about_dialog;
}
