/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "diagrampopupmenu.h"
#include "appfilemenu.h"
#include "diagramwindow.h"
#include "diagramarea.h"
#include "appwindow.h"
#include "appbuttons.h"
#include "dialogs.h"
#include "saver.h"
#include "diagram_graphic.h"
#include "i18n.h"
#include <stdio.h>

void
DiagramPopupMenu::save()
{
  string filename;

  filename = orderedset->getFilename();
  if (filename == "") saveasfile_dialog->exec();
  else {
    saveasfile_dialog->saveFile(filename);
    window->update();
  }
}

void
DiagramPopupMenu::saveAs()
{
  saveasfile_dialog->exec();
}

void
DiagramPopupMenu::undo()
{
  orderedset->undo();
  window->update();
}

void
DiagramPopupMenu::redo()
{
  orderedset->redo();
  window->update();
}

void
DiagramPopupMenu::cut()
{
  set<Element> elems;

  elems = area->getSelectedElements();
  if (elems.empty()) return;
  copy();
  area->removeSelectedElements();
  window->update();
}

void
DiagramPopupMenu::copy()
{
  OrderedSet *copy_os;
  set<Element> elems;
  set<Element>::iterator iter_s1, iter_s2;
  list<Element *> new_elems;
  list<Element *>::iterator iter_l1, iter_l2;

  elems = area->getSelectedElements();
  if (elems.empty()) return;
  copy_os = appwindow->getCopyOrderedSet();
  copy_os->clear();
  for (iter_s1 = elems.begin(); iter_s1 != elems.end(); iter_s1++)
    new_elems.
      push_back(copy_os->newElement(NoElement, (*iter_s1).name,
				    orderedset->getDiagram()->
				    getVertex(*iter_s1)));
  for (iter_s1 = elems.begin(), iter_l1 = new_elems.begin();
       iter_s1 != elems.end(); iter_s1++, iter_l1++)
    for (iter_s2 = elems.begin(), iter_l2 = new_elems.begin();
	 iter_s2 != elems.end(); iter_s2++, iter_l2++)
      if (orderedset->less(*iter_s1, *iter_s2, false) && *iter_l1 && *iter_l2)
	copy_os->orderElements(**iter_l1, **iter_l2);
}

void
DiagramPopupMenu::paste()
{
  OrderedSet *copy_os;
  set<Element> elems;
  set<Element>::iterator iter_s1, iter_s2;
  list<Element *> new_elems;
  list<Element *>::iterator iter_l1, iter_l2;

  copy_os = appwindow->getCopyOrderedSet();
  copy_os->getElements(elems);
  if (elems.empty()) return;
  orderedset->save_UndoRedo();
  for (iter_s1 = elems.begin(); iter_s1 != elems.end(); iter_s1++)
    new_elems.
      push_back(orderedset->newElement(NoElement, (*iter_s1).name,
				       copy_os->getDiagram()->
				       getVertex(*iter_s1)));
  for (iter_s1 = elems.begin(), iter_l1 = new_elems.begin();
       iter_s1 != elems.end(); iter_s1++, iter_l1++)
    for (iter_s2 = elems.begin(), iter_l2 = new_elems.begin();
	 iter_s2 != elems.end(); iter_s2++, iter_l2++)
      if (copy_os->less(*iter_s1, *iter_s2, false) && *iter_l1 && *iter_l2)
	orderedset->orderElements(**iter_l1, **iter_l2);
  window->update();
}

void
DiagramPopupMenu::clear()
{
  orderedset->save_UndoRedo();
  orderedset->clear();
  window->update();
}

void
DiagramPopupMenu::selectElements()
{
  appbuttons->selectButton(AppButtons::SELECT);
}

void
DiagramPopupMenu::selectInvert()
{
  area->invertSelectedElements();
  window->update();
}

void
DiagramPopupMenu::selectAll()
{
  area->clearSelectedElements();
  area->invertSelectedElements();
  window->update();
}

void
DiagramPopupMenu::selectNone()
{
  area->clearSelectedElements();
  window->update();
}

void
DiagramPopupMenu::selectMarked()
{
  area->selectMarkedElements();
  window->update();
}

void
DiagramPopupMenu::newDiagramView()
{
  int width, height;
  
  area->getMinDiagramSize(width, height);
  orderedsets->refSet(orderedset);
  new DiagramWindow(appwindow, orderedsets, orderedset, width, height);
}

void
DiagramPopupMenu::newElements()
{
  appbuttons->selectButton(AppButtons::NEW);
}

void
DiagramPopupMenu::renameElement()
{
  Vertex *vert_p;
  Element *elem = NULL;
  NewElementDialog *new_dialog;
  string name;
  
  vert_p = orderedset->getDiagram()->
    getVertexInBox(x_menu, y_menu, area->getDiagramVertexSize() / 2);
  if (vert_p) elem = orderedset->getDiagram()->getElement(*vert_p);
  if (elem) {
    new_dialog = new NewElementDialog(true);
    new_dialog->exec();
    name = new_dialog->getElementName();
    delete new_dialog;
    //if (!name.empty()) {
      orderedset->save_UndoRedo();
      orderedset->rename(*elem, name);
      window->update();
      //}
  }
}

void
DiagramPopupMenu::removeElements()
{
  appbuttons->selectButton(AppButtons::REMOVE);
}

void
DiagramPopupMenu::coverElements()
{
  appbuttons->selectButton(AppButtons::COVER);
}

void
DiagramPopupMenu::removeCover()
{
  appbuttons->selectButton(AppButtons::REMOVE_COVER);
}

void
DiagramPopupMenu::rearrangeDiagram()
{
  rearrangediagram_dialog->exec();
}

void
DiagramPopupMenu::markUpper()
{
  appbuttons->selectButton(AppButtons::MARK_UPPER);
}

void
DiagramPopupMenu::markLower()
{
  appbuttons->selectButton(AppButtons::MARK_LOWER);
}

void
DiagramPopupMenu::markMinimal()
{
  set<Element> s;

  orderedset->minimalElements(s, area->getSelectedElements());
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markMaximal()
{
  set<Element> s;

  orderedset->maximalElements(s, area->getSelectedElements());
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markLeast()
{
  set<Element> s;
  Element *elem;

  elem = orderedset->leastElement(area->getSelectedElements());
  if (elem) s.insert(*elem);
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markGreatest()
{
  set<Element> s;
  Element *elem;

  elem = orderedset->greatestElement(area->getSelectedElements());
  if (elem) s.insert(*elem);
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markLowerCone()
{
  set<Element> s;

  orderedset->lowerCone(s, area->getSelectedElements());
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markUpperCone()
{
  set<Element> s;

  orderedset->upperCone(s, area->getSelectedElements());
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markInfimum()
{
  set<Element> s;
  Element *elem;

  elem = orderedset->infimum(area->getSelectedElements());
  if (elem) s.insert(*elem);
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markSupremum()
{
  set<Element> s;
  Element *elem;

  elem = orderedset->supremum(area->getSelectedElements());
  if (elem) s.insert(*elem);
  area->markElements(s);
  window->update();
}

void
DiagramPopupMenu::markPaths()
{
  appbuttons->selectButton(AppButtons::MARK_PATHS);
}

void
DiagramPopupMenu::unmarkAll()
{
  area->clearMarkedElements();
  window->update();
}

void
DiagramPopupMenu::hideVertecesLabels()
{
  set<Element> elems;
  set<Element>::iterator iter_s;
  Vertex *vert = NULL;

  elems = area->getSelectedElements();
  if (elems.empty()) return;
  for (iter_s = elems.begin(); iter_s != elems.end(); iter_s++) {
    vert = orderedset->getDiagram()->getVertex(*iter_s);
    if (!vert) continue;
    vert->properties |= Vertex::HIDDEN_LABEL;
    orderedset->getDiagram()->updateVertex(*vert);
    delete vert; vert = NULL;
  }
  window->update();
}

void
DiagramPopupMenu::showVertecesLabels()
{
  set<Element> elems;
  set<Element>::iterator iter_s;
  Vertex *vert = NULL;

  elems = area->getSelectedElements();
  if (elems.empty()) return;
  for (iter_s = elems.begin(); iter_s != elems.end(); iter_s++) {
    vert = orderedset->getDiagram()->getVertex(*iter_s);
    if (!vert) continue;
    vert->properties &= (unsigned long)~Vertex::HIDDEN_LABEL;
    orderedset->getDiagram()->updateVertex(*vert);
    delete vert; vert = NULL;
  }
  window->update();
}

DiagramPopupMenu::DiagramPopupMenu(AppWindow *appwin, DiagramWindow *win, 
				   OrderedSets *oss, OrderedSet *os)
  : QPopupMenu(win)
{
  AppFileMenu *filemenu;
  QPopupMenu *editmenu, *selectmenu, *viewmenu, *datamenu, *data_markmenu,
    *diagrammenu;
  orderedsets = oss;
  orderedset = os;
  appwindow = appwin;
  window = win;
  area = NULL;
  appbuttons = appwin->getButtons();
  x_menu = y_menu = 0;
  // file
  filemenu = new AppFileMenu(appwin, oss);
  insertItem(QString::fromLocal8Bit(_("File")), filemenu);
  // save
  filemenu->insertItem(QString::fromLocal8Bit(_("Save")), this, 
		       SLOT(save()), CTRL+Key_S, -1, 3);
  // save as
  filemenu->insertItem(QString::fromLocal8Bit(_("Save As...")), this, 
		       SLOT(saveAs()), 0, -1, 4);
  // close
  filemenu->insertItem(QString::fromLocal8Bit(_("Close")), window,
		       SLOT(close()), CTRL+Key_W, -1, 6);
  // edit
  editmenu = new QPopupMenu(this);
  insertItem(QString::fromLocal8Bit(_("Edit")), editmenu);
  // tearoff
  editmenu->insertTearOffHandle();
  // undo
  editmenu->insertItem(QString::fromLocal8Bit(_("Undo")), this, 
		       SLOT(undo()), CTRL+Key_Z);
  // redo
  editmenu->insertItem(QString::fromLocal8Bit(_("Redo")), this, 
		       SLOT(redo()), CTRL+Key_R);
  // separator
  editmenu->insertSeparator();
  // cut
  editmenu->insertItem(QString::fromLocal8Bit(_("Cut")), this,
		       SLOT(cut()), CTRL+Key_X);
  // copy
  editmenu->insertItem(QString::fromLocal8Bit(_("Copy")), this, 
		       SLOT(copy()), CTRL+Key_C);
  // paste
  editmenu->insertItem(QString::fromLocal8Bit(_("Paste")), this,
		       SLOT(paste()), CTRL+Key_V);
  // separator
  editmenu->insertSeparator();
  // clear
  editmenu->insertItem(QString::fromLocal8Bit(_("Clear")), this,
		       SLOT(clear()), CTRL+Key_K);
  editmenu->show();
  // select
  selectmenu = new QPopupMenu(this);
  insertItem(QString::fromLocal8Bit(_("Select")), selectmenu);
  // tearoff
  selectmenu->insertTearOffHandle();
  // select elements
  selectmenu->insertItem
    (QString::fromLocal8Bit(_("Select elements")), this,
     SLOT(selectElements()), Key_S);
  // separator
  selectmenu->insertSeparator();
  // invert
  selectmenu->insertItem(QString::fromLocal8Bit(_("Invert")), this,
			 SLOT(selectInvert()), CTRL+Key_I);
  // all
  selectmenu->insertItem(QString::fromLocal8Bit(_("All")), this, 
			 SLOT(selectAll()), CTRL+Key_A);
  // none
  selectmenu->insertItem(QString::fromLocal8Bit(_("None")), this,
			 SLOT(selectNone()), SHIFT+CTRL+Key_A);
  // separator
  selectmenu->insertSeparator();
  // marked
  selectmenu->insertItem(QString::fromLocal8Bit(_("Marked")), this,
			 SLOT(selectMarked()), CTRL+Key_M);
  selectmenu->show();
  // view
  viewmenu = new QPopupMenu(this);
  insertItem(QString::fromLocal8Bit(_("View")), viewmenu);
  // tearoff
  viewmenu->insertTearOffHandle();
  // new diagram view
  viewmenu->insertItem(QString::fromLocal8Bit(_("New Diagram View")), this,
		       SLOT(newDiagramView()));
  viewmenu->show();
  // data
  datamenu = new QPopupMenu(this);
  insertItem(QString::fromLocal8Bit(_("Data")), datamenu);
  // tearoff
  datamenu->insertTearOffHandle();
  // insert elements
  datamenu->insertItem
    (QString::fromLocal8Bit(_("New elements")), this,
     SLOT(newElements()), Key_Insert);
  // insert elements
  datamenu->insertItem
    (QString::fromLocal8Bit(_("Rename element...")), this,
     SLOT(renameElement()));
  // remove elements
  datamenu->insertItem
    (QString::fromLocal8Bit(_("Remove elements")), this, 
     SLOT(removeElements()), Key_Delete);
  // cover two elements
  datamenu->insertItem
    (QString::fromLocal8Bit(_("Define cover x < y")), this, 
     SLOT(coverElements()), Key_C);
  // remove cover
  datamenu->insertItem
    (QString::fromLocal8Bit(_("Remove cover")), this,
     SLOT(removeCover()), SHIFT+Key_C);
  // separator
  datamenu->insertSeparator();
  // mark
  data_markmenu = new QPopupMenu(this);
  datamenu->insertItem(QString::fromLocal8Bit(_("Mark")), data_markmenu);
  // tearoff
  data_markmenu->insertTearOffHandle();
  // lower elements
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Lower elements")), this, 
     SLOT(markLower()), SHIFT+Key_L);
  // upper elements
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Upper elements")), this,
     SLOT(markUpper()), SHIFT+Key_U);
  // separator
  data_markmenu->insertSeparator();
  // minimal elements
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Minimal elements")), this, 
     SLOT(markMinimal()), SHIFT+Key_I);
  // maximal elements
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Maximal elements")), this, 
     SLOT(markMaximal()), SHIFT+Key_A);
  // least element
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Least element")), this,
     SLOT(markLeast()), SHIFT+Key_E);
  // greatest element
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Greatest element")), this, 
     SLOT(markGreatest()), SHIFT+Key_G);
  // lower cone
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Lower cone")), this,
     SLOT(markLowerCone()), SHIFT+Key_W);
  // upper cone
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Upper cone")), this,
     SLOT(markUpperCone()), SHIFT+Key_R);
  // infimum
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Infimum")), this,
     SLOT(markInfimum()), SHIFT+Key_F);
  // supremum
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Supremum")), this,
     SLOT(markSupremum()), SHIFT+Key_S);
  // separator
  data_markmenu->insertSeparator();
  // paths
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Paths...")), this,
     SLOT(markPaths()), SHIFT+Key_P);
  // separator
  data_markmenu->insertSeparator();
  // unmark all
  data_markmenu->insertItem
    (QString::fromLocal8Bit(_("Unmark all")), this,
     SLOT(unmarkAll()), SHIFT+Key_M);
  data_markmenu->show();
  datamenu->show();
  // diagram
  diagrammenu = new QPopupMenu(this);
  insertItem(QString::fromLocal8Bit(_("Diagram")), diagrammenu);
  // tearoff
  diagrammenu->insertTearOffHandle();
  // rearrange diagram
  diagrammenu->insertItem(QString::fromLocal8Bit(_("Rearrange diagram...")),
			  this, SLOT(rearrangeDiagram()), CTRL+Key_D);
  // separator
  diagrammenu->insertSeparator();
  // hide verteces labels
  diagrammenu->insertItem(QString::fromLocal8Bit(_("Hide verteces labels")),
			  this, SLOT(hideVertecesLabels()), CTRL+Key_H);
  // show verteces labels
  diagrammenu->insertItem(QString::fromLocal8Bit(_("Show verteces labels")),
			  this, SLOT(showVertecesLabels()), SHIFT+CTRL+Key_H);
  diagrammenu->show();
  saveasfile_dialog = new SaveAsFileDialog(orderedset, window, appwindow);
  rearrangediagram_dialog = new RearrangeDiagramDialog(orderedset, window,
						       appwindow);
  show();
}

void
DiagramPopupMenu::popup(int x, int y, int x_area, int y_area)
{
  x_menu = x; y_menu = y;
  // popup
  QPopupMenu::popup(area->mapToGlobal(QPoint(x_area, y_area)));
}

DiagramPopupMenu::modes_t
DiagramPopupMenu::getMode()
{
  switch (appbuttons->getSelectedButton()) {
  case AppButtons::SELECT : return MODE_SELECT;
  case AppButtons::NEW : return MODE_NEW;
  case AppButtons::REMOVE : return MODE_REMOVE;
  case AppButtons::COVER : return MODE_COVER;
  case AppButtons::REMOVE_COVER : return MODE_REMOVE_COVER;
  case AppButtons::MARK_UPPER : return MODE_MARK_UPPER;
  case AppButtons::MARK_LOWER : return MODE_MARK_LOWER;
  case AppButtons::MARK_PATHS : return MODE_MARK_PATHS;
  }
  return MODE_SELECT;
}
