/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "orderedset.h"
#include "dialogs.h"
#include "diagramarea.h"
#include "orderedsets.h"
#include "diagramwindow.h"
#include "loader.h"
#include "saver.h"
#include "i18n.h"
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qvbuttongroup.h>
#include <qhbuttongroup.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qhbox.h>
#include <qtabwidget.h>
#include <stdio.h>

MessageDialog::MessageDialog(QString title, QString message)
  : QMessageBox(title, message, QMessageBox::NoIcon, 
		QMessageBox::Ok | QMessageBox::Default, 
		QMessageBox::NoButton, QMessageBox::NoButton) // modal
{
  // disable shrink/growing
  adjustSize();
  setFixedSize(size());
}

QuestionDialog::QuestionDialog(QString title, QString message, 
			       QString question)
  : QMessageBox(title, message + "\n" + question, QMessageBox::NoIcon, 
		QMessageBox::Yes | QMessageBox::Default, 
		QMessageBox::No | QMessageBox::Escape, 
		QMessageBox::NoButton) // modal
{
  // disable shrink/growing
  adjustSize();
  setFixedSize(size());
}

void
NewElementDialog::accept()
{
  //if (edit->text() == "") return;
  QDialog::accept();
}

void
NewElementDialog::editChanged()
{
  //ok_button->setEnabled((edit->text() != ""));
}

NewElementDialog::NewElementDialog(bool rename_dialog)
  : QDialog(0, 0, true) // modal
{
  QLabel *label;
  QPushButton *cancel_button;
  QHBoxLayout *hbox;
  QVBoxLayout *vbox;

  if (!rename_dialog) setCaption(QString::fromLocal8Bit(_("New element")));
  else setCaption(QString::fromLocal8Bit(_("Rename element")));
  if (!rename_dialog) 
    label = 
      new QLabel(QString::fromLocal8Bit(_("New element's name: ")), this);
  else
    label = 
      new QLabel(QString::fromLocal8Bit(_("Element's new name: ")), this);
  // left alignment
  label->setAlignment(AlignLeft);
  label->show();
  edit = new QLineEdit(this);
  // enter
  connect(edit, SIGNAL(returnPressed()), this, SLOT(accept()));
  // changed
  connect(edit, SIGNAL(textChanged(const QString &)), this, 
	  SLOT(editChanged()));
  // focus
  edit->setFocus();
  edit->show();
  ok_button = new QPushButton(QString::fromLocal8Bit(_("OK")), this);
  // can default
  ok_button->setAutoDefault(true);
  // disabled
  //ok_button->setEnabled(false);
  // default
  ok_button->setDefault(true);
  connect(ok_button, SIGNAL(clicked()), this, SLOT(accept()));
  ok_button->show();
  cancel_button = new QPushButton(QString::fromLocal8Bit(_("Cancel")), this);
  // can default
  cancel_button->setAutoDefault(true);
  connect(cancel_button, SIGNAL(clicked()), this, SLOT(reject()));
  cancel_button->show();
  hbox = new QHBoxLayout();
  hbox->addWidget(ok_button);
  hbox->addWidget(cancel_button);
  vbox = new QVBoxLayout(this, 10);
  vbox->addWidget(label);
  vbox->addWidget(edit);
  vbox->addLayout(hbox);
  // disable shrink/growing
  vbox->setResizeMode(QLayout::Fixed);
}

string
NewElementDialog::getElementName()
{
  return (const char *)edit->text();
}

void
OpenSaveFileDialog::fileChanged()
{
  // disable ok button
}

OpenSaveFileDialog::OpenSaveFileDialog(QWidget *parent)
  : QFileDialog(parent, 0, false) // modeless
{
  // disable ok button
  // enter on file
  // file changed
}

string
OpenSaveFileDialog::getFilename()
{
  string filename;
 
  filename = selectedFile();
  if (filename == "") return "";
  return filename;
}

void
OpenFileDialog::accept()
{
  OrderedSet *os;
  
  if (openFile(getFilename(), os)) {
    new DiagramWindow(appwindow, orderedsets, os);
    QFileDialog::accept();
  }
}

OpenFileDialog::OpenFileDialog(AppWindow *appwin, OrderedSets *oss)
  : OpenSaveFileDialog((QWidget *)appwin)
{ 
  char type[30] = "";
  QStringList types(QString::fromLocal8Bit(_("All files (*)")));

  appwindow = appwin;
  orderedsets = oss;
  setCaption(QString::fromLocal8Bit(_("Load set")));
  sprintf(type, QString::fromLocal8Bit(_("LAT (%s) files (*.lat)")),
	  PROG_NAME);
  types.append(type);
  setFilters(types);
}

bool
OpenFileDialog::openFile(string filename, OrderedSet *&os)
{
  FileLoader *loader;
  FileLoader::error_t load_err;
  MessageDialog *message_dialog;
  QString filetype;
  
  if (filename == "") return false;
  loader = new FileLoader(orderedsets);
  filetype = selectedFilter();
  if (filetype.contains(".lat"))
    load_err = loader->load(filename, FileLoader::XML, os);
  else load_err = loader->load(filename, FileLoader::NOTYPE, os);
  delete loader;
  if (load_err == FileLoader::ETYPE) {
    message_dialog = 
      new MessageDialog(QString::fromLocal8Bit(_("Load failed!")), 
			QString::fromLocal8Bit(filename.c_str()) + ": " + 
			QString::fromLocal8Bit(_("Unknown file type.")));
    message_dialog->exec();
    delete message_dialog;
  }
  else if (load_err == FileLoader::EACCESS) {
    message_dialog = 
      new MessageDialog(QString::fromLocal8Bit(_("Load failed!")), 
			QString::fromLocal8Bit(filename.c_str()) + " " + 
			QString::fromLocal8Bit(_("is not present or is unreadable.")));
    message_dialog->exec();
    delete message_dialog;
  }
  else if (load_err == FileLoader::ELOAD) {
    message_dialog = 
      new MessageDialog(QString::fromLocal8Bit(_("Load failed!")), 
			QString::fromLocal8Bit(filename.c_str()) + ": " + 
			QString::fromLocal8Bit(_("Load error.")));
    message_dialog->exec();
    delete message_dialog;
  }
  return (load_err == FileLoader::EOK);
}

void
SaveAsFileDialog::accept()
{
  string filename;
  FILE *file;
  QuestionDialog *question_dialog;
  bool answer = true;
  
  filename = getFilename();
  file = fopen(filename.c_str(), "r");
  if (file) {
    question_dialog = 
      new QuestionDialog(QString::fromLocal8Bit(_("File exists!")), 
			 QString::fromLocal8Bit(filename.c_str()) + " " + 
			 QString::fromLocal8Bit(_("exists.")),
			 QString::fromLocal8Bit(_("Overwrite?")));
    answer = question_dialog->exec();
    delete question_dialog;
    fclose(file);
  }
  if (answer && saveFile(filename)) {
    diagramwindow->update();
    QFileDialog::accept();
  }
}

SaveAsFileDialog::SaveAsFileDialog(OrderedSet *os, DiagramWindow *win,
				   AppWindow *appwin)
  : OpenSaveFileDialog((QWidget *)appwin)
{
  char type[30] = "";
  QStringList types(QString::fromLocal8Bit(_("All files (*)")));

  orderedset = os;
  diagramwindow = win;
  setCaption(QString::fromLocal8Bit(_("Save set")));
  sprintf(type, QString::fromLocal8Bit(_("LAT (%s) files (*.lat)")),
	  PROG_NAME);
  types.append(type);
  types.append(QString::fromLocal8Bit(_("MetaPost files (*.mp)")));
  types.append(QString::fromLocal8Bit(_("Encapsulated PostScript files (*.eps)")));
  setFilters(types);
}

bool
SaveAsFileDialog::saveFile(string filename)
{
  FileSaver *saver;
  FileSaver::error_t save_err;
  MessageDialog *message_dialog;
  QString filetype;
  
  if (filename == "") return false;
  saver = new FileSaver(orderedset);
  filetype = selectedFilter();
  if (filetype.contains(".lat"))
    save_err = saver->save(filename, FileSaver::XML);
  else if (filetype.contains(".mp"))
    save_err = saver->save(filename, FileSaver::MetaPost);
  else if (filetype.contains(".eps"))
    save_err = saver->save(filename, FileSaver::PostScript);
  else save_err = saver->save(filename, FileSaver::NOTYPE);
  delete saver;
  if (save_err == FileSaver::ETYPE) {
    message_dialog = 
      new MessageDialog(QString::fromLocal8Bit(_("Save failed!")), 
			QString::fromLocal8Bit(filename.c_str()) + ": " + 
			QString::fromLocal8Bit(_("Unknown file type.")));
    message_dialog->exec();
    delete message_dialog;
  }
  else if (save_err == FileSaver::EACCESS) {
    message_dialog = 
      new MessageDialog(QString::fromLocal8Bit(_("Save failed!")), 
			QString::fromLocal8Bit(filename.c_str()) + " " + 
			QString::fromLocal8Bit(_("is unwritable.")));
    message_dialog->exec();
    delete message_dialog;
  }
  else if (save_err == FileSaver::ESAVE) {
    message_dialog = 
      new MessageDialog(QString::fromLocal8Bit(_("Save failed!")), 
			QString::fromLocal8Bit(filename.c_str()) + ": " + 
			QString::fromLocal8Bit(_("Save error.")));
    message_dialog->exec();
    delete message_dialog;
  }
  return (save_err == FileSaver::EOK);
}

bool
NewSetDialog::checkValue(QString val)
{
  width = 0;
  sscanf(val, "%i", &width);
  if (width <= 0) return false;
  return true;
}

void
NewSetDialog::accept()
{
  OrderedSet *os;

  width = height = 0;
  sscanf(edit_w->text(), "%i", &width);
  sscanf(edit_h->text(), "%i", &height);
  if ((width <= 0) || (height <= 0)) return;
  QDialog::accept();
  if (tree_button->isChecked())
    os = orderedsets->newSet(OrderedSet::DATA_TREE);
  else if (list_button->isChecked())
    os = orderedsets->newSet(OrderedSet::DATA_LIST);
  else if (array_button->isChecked())
    os = orderedsets->newSet(OrderedSet::DATA_ARRAY);
  new DiagramWindow(appwindow, orderedsets, os, width, height);
}

void
NewSetDialog::sizeChanged()
{
  ok_button->setEnabled(checkValue(edit_w->text()) && 
			checkValue(edit_h->text()));
}

void
NewSetDialog::reset()
{
  char w[5] = "", h[5] = "";

  array_button->setChecked(true);
  list_button->setChecked(false);
  tree_button->setChecked(false);
  width = height = 200;
  sprintf(w, "%i", width);
  edit_w->setText(w);
  sprintf(h, "%i", height);
  edit_h->setText(h);
}

NewSetDialog::NewSetDialog(AppWindow *appwin, OrderedSets *oss)
  : QDialog((QWidget *)appwin, 0, false) // modeless
{
  QVButtonGroup *vbuttongroup;
  QPushButton *cancel_button, *reset_button;
  QHBoxLayout *aarea;
  QVBoxLayout *vbox;
  QHBox *width_hbox, *height_hbox;
  QHGroupBox *size_hgroup;
  QLabel *label_w, *label_h;

  appwindow = appwin;
  orderedsets = oss;
  setCaption(QString::fromLocal8Bit(_("New set")));
  vbuttongroup = 
    new QVButtonGroup(QString::fromLocal8Bit(_("Internal data representation")), this);
  vbuttongroup->show();
  array_button = 
    new QRadioButton(QString::fromLocal8Bit(_("Two-dimensional array")),
		     vbuttongroup);
  // focus
  array_button->setFocus();
  array_button->show();
  list_button = 
    new QRadioButton(QString::fromLocal8Bit(_("List of lists")),
		     vbuttongroup);
  list_button->show();
  tree_button = 
    new QRadioButton(QString::fromLocal8Bit(_("Red-Black tree")),
		     vbuttongroup);
  tree_button->show();
  size_hgroup = 
    new QHGroupBox(QString::fromLocal8Bit(_("Minimal diagram size")), this);
  size_hgroup->show();
  width_hbox = new QHBox(size_hgroup);
  width_hbox->setSpacing(5);
  width_hbox->show();
  label_w = 
    new QLabel(QString::fromLocal8Bit(_("Width: ")), width_hbox);
  // left alignment
  label_w->setAlignment(AlignLeft);
  label_w->show();
  edit_w = new QLineEdit(width_hbox);
  // max length
  edit_w->setMaxLength(4);
  // changed
  connect(edit_w, SIGNAL(textChanged(const QString &)), this, 
	  SLOT(sizeChanged()));
  edit_w->show();
  size_hgroup->addSpace(10);
  height_hbox = new QHBox(size_hgroup);
  height_hbox->setSpacing(5);
  height_hbox->show();
  label_h = 
    new QLabel(QString::fromLocal8Bit(_("Height: ")), height_hbox);
  // left alignment
  label_h->setAlignment(AlignLeft);
  label_h->show();
  edit_h = new QLineEdit(height_hbox);
  // max length
  edit_h->setMaxLength(4);
  // changed
  connect(edit_h, SIGNAL(textChanged(const QString &)), this, 
	  SLOT(sizeChanged()));
  edit_h->show();
  ok_button = new QPushButton(QString::fromLocal8Bit(_("OK")), this);
  // can default
  ok_button->setAutoDefault(true);
  // default
  ok_button->setDefault(true);
  connect(ok_button, SIGNAL(clicked()), this, SLOT(accept()));
  ok_button->show();
  reset_button = new QPushButton(QString::fromLocal8Bit(_("Reset")), this);
  // can default
  reset_button->setAutoDefault(true);
  connect(reset_button, SIGNAL(clicked()), this, SLOT(reset()));
  reset_button->show();
  cancel_button = new QPushButton(QString::fromLocal8Bit(_("Cancel")), this);
  // can default
  cancel_button->setAutoDefault(true);
  connect(cancel_button, SIGNAL(clicked()), this, SLOT(reject()));
  cancel_button->show();
  aarea = new QHBoxLayout();
  aarea->addWidget(ok_button);
  aarea->addWidget(reset_button);
  aarea->addWidget(cancel_button);
  vbox = new QVBoxLayout(this, 10);
  vbox->addWidget(vbuttongroup);
  vbox->addWidget(size_hgroup);
  vbox->addLayout(aarea);
  // disable shrink/growing
  vbox->setResizeMode(QLayout::Fixed);
  reset();
}

void
RearrangeDiagramDialog::rearrange()
{
  int num = 0;
  bool ok = true;
  MessageDialog message_dialog(QString::fromLocal8Bit(_("Warning!")), 
			       QString::fromLocal8Bit(_("Maximal width of the Coffman-Graham layering must be set to a positive number.")));
  
  num = tabwidget->currentPageIndex();
  if (num == 0) {
    generation_params.generation_method = Diagram_Graphic::DIAGRAM_LEVEL;
    generation_params.dle_fmin = level_min_button->isChecked();
  }
  else if (num == 1) {
    generation_params.generation_method = 
      Diagram_Graphic::DIAGRAM_GEOMETRIC;
    generation_params.dg_fgreat = geom_great_button->isChecked();
  }
  else if (num == 2) {
    generation_params.generation_method = Diagram_Graphic::DIAGRAM_LAYER;
    generation_params.dla_fmax = layer_max_button->isChecked();
    if (longest_path_button->isChecked())
      generation_params.dla_l = Diagram_Graphic::DLA_L_LONGEST_PATH;
    else if (coffman_graham_button->isChecked())
      generation_params.dla_l = Diagram_Graphic::DLA_L_COFFMAN_GRAHAM;
    sscanf(edit->text(), "%i", &num);
    if (num <= 0) { message_dialog.exec(); ok = false; }
    generation_params.dla_cgw = num;
    if (tree_button->isChecked())
      generation_params.dla_dsdt = Diagram_Graphic::DLA_DD_TREE;
    else if (list_button->isChecked())
      generation_params.dla_dsdt = Diagram_Graphic::DLA_DD_LIST;
    else if (array_button->isChecked())
      generation_params.dla_dsdt = Diagram_Graphic::DLA_DD_ARRAY;
    if (adjacent_exchange_button->isChecked())
      generation_params.dla_cr = Diagram_Graphic::DLA_CR_ADJACENT_EXCHANGE;
    else if (split_button->isChecked())
      generation_params.dla_cr = Diagram_Graphic::DLA_CR_SPLIT;
    else if (barycenter_button->isChecked())
      generation_params.dla_cr = Diagram_Graphic::DLA_CR_BARYCENTER;
    else if (median_button->isChecked())
      generation_params.dla_cr = Diagram_Graphic::DLA_CR_MEDIAN;
  }
  if (ok) {
    orderedset->save_UndoRedo();
    orderedset->getDiagram()->generateDiagram(generation_params);
    diagramwindow->update();
  }
}

RearrangeDiagramDialog::RearrangeDiagramDialog(OrderedSet *os, 
					       DiagramWindow *win,
					       AppWindow *appwin)
  : QDialog((QWidget *)appwin, 0, false) // modeless
{
  QVButtonGroup *dsdt_vbuttongroup;
  QHButtonGroup *l_hbuttongroup;
  QButtonGroup *cr_buttongroup;
  QPushButton *rearrange_button, *cancel_button;
  QHBoxLayout *aarea;
  QHBox *cgw_hbox;
  QVBoxLayout *vbox;
  QVGroupBox *level_vgroup, *geom_vgroup, *layer_vgroup;
  QLabel *label;
  char num[5] = "";

  orderedset = os;
  diagramwindow = win;
  generation_params = orderedset->getDiagram()->getGenerationParams();
  setCaption(QString::fromLocal8Bit(_("Rearrange diagram")));
  level_vgroup = 
    new QVGroupBox(QString::fromLocal8Bit(_("Level method parameters")),
		   this);
  level_vgroup->show();
  level_min_button = 
    new QCheckBox(QString::fromLocal8Bit(_("Generate diagram starting with minimal elements")), level_vgroup);
  level_min_button->show();
  if (generation_params.dle_fmin) level_min_button->setChecked(true);
  geom_vgroup = 
    new QVGroupBox(QString::fromLocal8Bit(_("Geometric method parameters")),
		   this);
  geom_vgroup->show();
  geom_great_button = 
    new QCheckBox(QString::fromLocal8Bit(_("Generate diagram starting with the greatest element")), geom_vgroup);
  geom_great_button->show();
  if (generation_params.dg_fgreat) geom_great_button->setChecked(true);
  layer_vgroup = 
    new QVGroupBox(QString::fromLocal8Bit(_("Layer method parameters")),
		   this);
  layer_vgroup->show();
  layer_max_button = 
    new QCheckBox(QString::fromLocal8Bit(_("Generate diagram starting with maximal elements")), layer_vgroup);
  layer_max_button->show();
  if (generation_params.dla_fmax) layer_max_button->setChecked(true);
  l_hbuttongroup = 
    new QHButtonGroup(QString::fromLocal8Bit(_("Layering")), layer_vgroup);
  l_hbuttongroup->show();
  longest_path_button = 
    new QRadioButton(QString::fromLocal8Bit(_("Longest Path")),
		     l_hbuttongroup);
  longest_path_button->show();
  coffman_graham_button = 
    new QRadioButton(QString::fromLocal8Bit(_("Coffman-Graham")),
		     l_hbuttongroup);
  coffman_graham_button->show();
  switch (generation_params.dla_l) {
  case Diagram_Graphic::DLA_L_LONGEST_PATH: 
    longest_path_button->setChecked(true); break;
  case Diagram_Graphic::DLA_L_COFFMAN_GRAHAM: 
    coffman_graham_button->setChecked(true); break;
  }
  cgw_hbox = new QHBox(layer_vgroup);
  cgw_hbox->setSpacing(5);
  cgw_hbox->show();
  label = 
    new QLabel(QString::fromLocal8Bit(_("Maximal width of the Coffman-Graham layering: ")), cgw_hbox);
  // left alignment
  label->setAlignment(AlignLeft);
  label->show();
  sprintf(num, "%i", generation_params.dla_cgw);
  edit = new QLineEdit(num, cgw_hbox);
  // max length
  edit->setMaxLength(4);
  edit->show();
  dsdt_vbuttongroup = 
    new QVButtonGroup(QString::fromLocal8Bit(_("Dummy chains internal data representation")), layer_vgroup);
  dsdt_vbuttongroup->show();
  array_button = 
    new QRadioButton(QString::fromLocal8Bit(_("Two-dimensional array")),
		     dsdt_vbuttongroup);
  array_button->show();
  list_button = 
    new QRadioButton(QString::fromLocal8Bit(_("List of lists")),
		     dsdt_vbuttongroup);
  list_button->show();
  tree_button = 
    new QRadioButton(QString::fromLocal8Bit(_("Red-Black tree")),
		     dsdt_vbuttongroup);
  tree_button->show();
  switch (generation_params.dla_dsdt) {
  case Diagram_Graphic::DLA_DD_TREE: 
    tree_button->setChecked(true); break;
  case Diagram_Graphic::DLA_DD_LIST: 
    list_button->setChecked(true); break;
  case Diagram_Graphic::DLA_DD_ARRAY: 
    array_button->setChecked(true); break;
  }
  cr_buttongroup = 
    new QButtonGroup(2, Horizontal, 
		     QString::fromLocal8Bit(_("Crossing reduction")),
		     layer_vgroup);
  cr_buttongroup->show();
  adjacent_exchange_button = 
    new QRadioButton(QString::fromLocal8Bit(_("Adjacent-Exchange sorting")),
		     cr_buttongroup);
  adjacent_exchange_button->show();
  split_button = 
    new QRadioButton(QString::fromLocal8Bit(_("Split sorting")),
		     cr_buttongroup);
  split_button->show();
  barycenter_button =
    new QRadioButton(QString::fromLocal8Bit(_("Barycenter")), cr_buttongroup);
  barycenter_button->show();
  median_button = 
    new QRadioButton(QString::fromLocal8Bit(_("Median")), cr_buttongroup);
  median_button->show();
  switch (generation_params.dla_cr) {
  case Diagram_Graphic::DLA_CR_ADJACENT_EXCHANGE: 
    adjacent_exchange_button->setChecked(true); break;
  case Diagram_Graphic::DLA_CR_SPLIT: 
    split_button->setChecked(true); break;
  case Diagram_Graphic::DLA_CR_BARYCENTER: 
    barycenter_button->setChecked(true); break;
  case Diagram_Graphic::DLA_CR_MEDIAN: 
    median_button->setChecked(true); break;
  }
  rearrange_button = 
    new QPushButton(QString::fromLocal8Bit(_("Rearrange")), this);
  // can default
  rearrange_button->setAutoDefault(true);
  // default
  rearrange_button->setDefault(true);
  connect(rearrange_button, SIGNAL(clicked()), this, SLOT(rearrange()));
  rearrange_button->show();
  cancel_button = new QPushButton(QString::fromLocal8Bit(_("Cancel")), this);
  // can default
  cancel_button->setAutoDefault(true);
  connect(cancel_button, SIGNAL(clicked()), this, SLOT(reject()));
  cancel_button->show();
  aarea = new QHBoxLayout();
  aarea->addWidget(rearrange_button);
  aarea->addWidget(cancel_button);
  tabwidget = new QTabWidget(this);
  tabwidget->setMargin(10);
  // tabs position
  tabwidget->setTabPosition(QTabWidget::Top);
  // popup
  tabwidget->addTab(level_vgroup, QString::fromLocal8Bit(_("Level method")));
  tabwidget->addTab(geom_vgroup, 
		    QString::fromLocal8Bit(_("Geometric method")));
  tabwidget->addTab(layer_vgroup, QString::fromLocal8Bit(_("Layer method")));
  // selected method
  switch (generation_params.generation_method) {
  case Diagram_Graphic::DIAGRAM_LEVEL: 
    tabwidget->setCurrentPage(0); break;
  case Diagram_Graphic::DIAGRAM_GEOMETRIC: 
    tabwidget->setCurrentPage(1); break;
  case Diagram_Graphic::DIAGRAM_LAYER: 
    tabwidget->setCurrentPage(2); break;
  }
  tabwidget->show();
  vbox = new QVBoxLayout(this, 10);
  vbox->addWidget(tabwidget);
  vbox->addLayout(aarea);
  // disable shrink/growing
  vbox->setResizeMode(QLayout::Fixed);
}

void
SelectPathDialog::select()
{
  int pos = 0, i;
  list<set<Element> >::iterator iter_l;

  pos = paths_listbox->currentItem();
  for (iter_l = element_paths.begin(), i = 0; i < pos; iter_l++, i++);
  diagramarea->markElements(*iter_l);
  diagramarea->update();
}

SelectPathDialog::SelectPathDialog(DiagramArea *area)
  : QDialog(area, 0, false) // modeless
{
  QPushButton *cancel_button;
  QHBoxLayout *aarea;
  QVBoxLayout *vbox;
  QLabel *label;

  diagramarea = area;
  setCaption(QString::fromLocal8Bit(_("Select path")));
  vbox = new QVBoxLayout(this, 10);
  label = new QLabel(QString::fromLocal8Bit(_("Found paths: ")), this);
  // left alignment
  label->setAlignment(AlignLeft);
  label->show();
  paths_listbox = new QListBox(this);
  paths_listbox->setSelectionMode(QListBox::Single);
  connect(paths_listbox, SIGNAL(selectionChanged()), this, SLOT(select()));
  // automatic scrollbars
  paths_listbox->setVScrollBarMode(QScrollView::Auto);
  paths_listbox->setHScrollBarMode(QScrollView::Auto);
  paths_listbox->setMinimumSize(250, 165);
  paths_listbox->show();
  cancel_button = new QPushButton(QString::fromLocal8Bit(_("Cancel")), this);
  // can default
  cancel_button->setAutoDefault(true);
  // default
  cancel_button->setDefault(true);
  connect(cancel_button, SIGNAL(clicked()), this, SLOT(reject()));
  cancel_button->show();
  aarea = new QHBoxLayout();
  aarea->addWidget(cancel_button);
  vbox->addWidget(label);
  vbox->addWidget(paths_listbox);
  vbox->addLayout(aarea);
  // disable shrinking
}

void
SelectPathDialog::setPaths(list<set<Element> > &paths)
{
  list<set<Element> >::iterator iter_l;
  set<Element>::iterator iter_s;
  string text;

  element_paths = paths;
  paths_listbox->clear();
  for (iter_l = paths.begin(); iter_l != paths.end(); iter_l++) {
    text = "";
    for (iter_s = (*iter_l).begin(); iter_s != (*iter_l).end(); iter_s++) {
      if (iter_s != (*iter_l).begin()) text += " - ";
      text += (*iter_s).name;
    }
    if (text != "") paths_listbox->insertItem(text.c_str());
  }
  paths_listbox->setCurrentItem(0);
}

AboutDialog::AboutDialog()
  : QDialog(0, 0, false) // modeless
{
  QVBoxLayout *vbox;
  QLabel *label;
  QPushButton *ok_button;
  QHBoxLayout *aarea;

  setCaption(QString::fromLocal8Bit(_("About the")) + " " + PROG_NAME);
  vbox = new QVBoxLayout(this, 10);
  label = new QLabel(QString(PROG_NAME) + "\n\n" + 
		     QString::fromLocal8Bit(_("Visualize lattices and ordered sets!")) + "\n\n" + QString::fromLocal8Bit(_("Version")) + " " + VERSION +
		     "\n\n" + QString::fromLocal8Bit(_("Author: ")) +
		     "Jan Outrata" + "\n\n" + 
		     "Email: outrataj@phoenix.inf.upol.cz", this);
  // center alignment
  label->setAlignment(AlignCenter);
  vbox->addWidget(label);
  ok_button = new QPushButton(QString::fromLocal8Bit(_("OK")), this);
  // can default
  ok_button->setAutoDefault(true);
  // default
  ok_button->setDefault(true);
  connect(ok_button, SIGNAL(clicked()), this, SLOT(accept()));
  ok_button->show();
  aarea = new QHBoxLayout();
  aarea->addWidget(ok_button);
  vbox->addLayout(aarea);
  // disable shrinking
  vbox->setResizeMode(QLayout::Fixed);
}
