/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// classes of the message, question, element insert,
// file save as and open dialogs

#ifndef __DIALOGS_H__
#define __DIALOGS_H__

#include <qmessagebox.h>
#include <qfiledialog.h>
#include <string>
#include <list>
#include "diagram_graphic.h"

using namespace std;

class OrderedSet;
class OrderedSets;
class DiagramArea;
class AppWindow;
class DiagramWindow;

class QLineEdit;
class QPushButton;
class QRadioButton;
class QCheckBox;
class QTabWidget;

class MessageDialog : public QMessageBox {
 public:
  MessageDialog(QString title, QString message);
  ~MessageDialog() {}
  bool exec() { return (QMessageBox::exec() == QMessageBox::Ok); }
};

class QuestionDialog : public QMessageBox {
 public:
  QuestionDialog(QString title, QString message, QString question);
  ~QuestionDialog() {}
  bool exec() { return (QMessageBox::exec() == QMessageBox::Yes); }
};

class NewElementDialog : public QDialog {
  Q_OBJECT
    protected:
  QPushButton *ok_button;
  QLineEdit *edit;
  protected slots:
    void accept();
  void editChanged();
 public:
  NewElementDialog(bool rename_dialog = false);
  ~NewElementDialog() {}
  bool exec() { return (QDialog::exec() == QDialog::Accepted); }
  string getElementName();
};

class OpenSaveFileDialog : public QFileDialog {
  Q_OBJECT
    protected:
  protected slots:
    virtual void accept() = 0;
  void fileChanged();
 public:
  OpenSaveFileDialog(QWidget *parent);
  virtual ~OpenSaveFileDialog() {}
  bool exec() { QFileDialog::show(); return true; }
  string getFilename();
};

class OpenFileDialog : public OpenSaveFileDialog {
 protected:
  AppWindow *appwindow;
  OrderedSets *orderedsets;
  void accept();
 public:
  OpenFileDialog(AppWindow *appwin, OrderedSets *oss);
  ~OpenFileDialog() {}
  bool openFile(string filename, OrderedSet *&os);
};

class SaveAsFileDialog : public OpenSaveFileDialog {
 protected:
  OrderedSet *orderedset;
  DiagramWindow *diagramwindow;
  void accept();
 public:
  SaveAsFileDialog(OrderedSet *os, DiagramWindow *win, AppWindow *appwin);
  ~SaveAsFileDialog() {}
  bool saveFile(string filename);
};

class NewSetDialog : public QDialog {
  Q_OBJECT
    protected:
  AppWindow *appwindow;
  OrderedSets *orderedsets;
  int width, height;
  QPushButton *ok_button;
  QRadioButton *tree_button, *list_button, *array_button;
  QLineEdit *edit_w, *edit_h;
  bool checkValue(QString val);
  protected slots:
    void accept();
  void sizeChanged();
  void reset();
 public:
  NewSetDialog(AppWindow *appwin, OrderedSets *oss);
  ~NewSetDialog() {}
  bool exec() { QDialog::show(); return true; }
};

class RearrangeDiagramDialog : public QDialog {
  Q_OBJECT
    protected:
  OrderedSet *orderedset;
  DiagramWindow *diagramwindow;
  Diagram_Graphic::generationParams_t generation_params;
  QTabWidget *tabwidget;
  QRadioButton *longest_path_button, *coffman_graham_button, *array_button,
    *list_button, *tree_button, *adjacent_exchange_button, *split_button,
    *barycenter_button, *median_button;
  QCheckBox *level_min_button, *geom_great_button, *layer_max_button;
  QLineEdit *edit;
  protected slots:
    void rearrange();
 public:
  RearrangeDiagramDialog(OrderedSet *os, DiagramWindow *win,
			 AppWindow *appwin);
  ~RearrangeDiagramDialog() {}
  bool exec() { QDialog::show(); return true; }
};

class SelectPathDialog: public QDialog {
  Q_OBJECT
    protected:
  list<set<Element> > element_paths;
  DiagramArea *diagramarea;
  QListBox *paths_listbox;
  protected slots:
    void select();
 public:
  SelectPathDialog(DiagramArea *area);
  ~SelectPathDialog() {}
  void setPaths(list<set<Element> > &paths);
  bool exec() { QDialog::show(); return true; }
};

class AboutDialog: public QDialog {
 public:
  AboutDialog();
  bool exec() { QDialog::show(); return true; }
};

#endif
