/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// abstract class of the ordered set data

#ifndef __ORDERDATA_H__
#define __ORDERDATA_H__

#include "element.h"
#include <set>
#include <list>

class OrderData {
 public:
  OrderData() {}
  virtual ~OrderData() {}
  virtual OrderData *getCopy() = 0;
  // returns: the object copy (or NULL)
  virtual void clear() = 0;
  // clears the set
  virtual bool newElement(ElementId x) = 0;
  // inserts new element x
  // returns: if successful (not)
  virtual bool removeElement(ElementId x) = 0;
  // removes element x
  // returns: if successful (not)
  virtual bool orderElements(ElementId x, ElementId y) = 0;
  // orders elements x, y such, that x is less than y
  // returns: if successful (not)
  virtual bool removeOrder(ElementId x, ElementId y) = 0;
  // removess order of elements x, y
  // returns: if successful (not)
  virtual bool less(ElementId x, ElementId y, bool transitivity = true) = 0;
  // if transitivity is false, doesn't iterate transitively
  // returns: if element x is less than element y
  virtual void getElements(set<ElementId> &s) = 0;
  // returns: all elements in s
  virtual void upperElements(set<ElementId> &s, ElementId x) = 0;
  // returns: all upper elements of element x in s
  virtual void lowerElements(set<ElementId> &s, ElementId x) = 0;
  // returns: all lower elements of element x in s
  virtual void minimalElements(set<ElementId> &s, set<ElementId> ss) = 0;
  // returns: minimal elements of the subset ss in s
  virtual void maximalElements(set<ElementId> &s, set<ElementId> ss) = 0;
  // returns: maximal elements of the subset ss in s  
  virtual ElementId leastElement(set<ElementId> ss) = 0;
  // returns: the least element of the subset ss, or NoElement
  virtual ElementId greatestElement(set<ElementId> ss) = 0;
  // returns: the greatest element of the subset ss, or NoElement
  virtual void lowerCone(set<ElementId> &s, set<ElementId> ss) = 0;
  // returns: lower cone of the subset ss in s
  virtual void upperCone(set<ElementId> &s, set<ElementId> ss) = 0;
  // returns: upper cone of the subset ss in s
  virtual ElementId infimum(set<ElementId> ss) = 0;
  // returns: infimum element of the subset ss, or NoElement
  virtual ElementId supremum(set<ElementId> ss) = 0;
  // returns: supremum element of the subset ss, or NoElement
  virtual void getPaths(list<set<ElementId> > &l, ElementId x, ElementId y)
    = 0;
  // returns: the list of all paths from element x to element y in l
};

#endif
