/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "orderedsets.h"
#include <stdio.h>

OrderedSets::~OrderedSets()
{
  set<RefOrderedSet_t>::iterator iter_s;

  for (iter_s = sets.begin(); iter_s != sets.end(); iter_s++)
    if ((*iter_s).orderedset) delete (*iter_s).orderedset;
  sets.clear();
}

OrderedSet *
OrderedSets::newSet(OrderedSet::data_t data_type)
{
  OrderedSet *new_set = NULL;
  OrderedSetId new_id;
  RefOrderedSet_t refset;
  set<RefOrderedSet_t>::iterator iter_s;
  
  for (iter_s = sets.begin(), new_id = 1; iter_s != sets.end(); 
       iter_s++, new_id++)
    if ((*iter_s).orderedset) if (*((*iter_s).orderedset) != new_id) break;
  new_set = new OrderedSet(data_type, new_id);
  if (!new_set) return NULL;
  refset.orderedset = new_set;
  refset.refcounter = new int; *(refset.refcounter) = 1;
  sets.insert(refset);
  return new_set;
}

bool
OrderedSets::refSet(OrderedSet *s)
{
  RefOrderedSet_t refset;
  set<RefOrderedSet_t>::iterator iter_s;
  
  refset.orderedset = s;
  iter_s = sets.find(refset);
  if (iter_s == sets.end()) return false;
  (*((*iter_s).refcounter))++;
  return true;
}

bool
OrderedSets::unrefSet(OrderedSet *s)
{
  RefOrderedSet_t refset;
  set<RefOrderedSet_t>::iterator iter_s;
  
  refset.orderedset = s;
  iter_s = sets.find(refset);
  if (iter_s == sets.end()) return false;
  if (--(*((*iter_s).refcounter)) == 0) {
    if ((*iter_s).orderedset) delete (*iter_s).orderedset;
    delete (*iter_s).refcounter;
    sets.erase(iter_s);
  }
  return true;
}

bool
OrderedSets::areAllSetsSaved()
{
  set<RefOrderedSet_t>::iterator iter_s;
  
  for (iter_s = sets.begin(); iter_s != sets.end(); iter_s++)
    if ((*iter_s).orderedset)
      if (!((*iter_s).orderedset)->isFileSaved()) return false;
  return true;
}
